% neurosky_read_thinkgear - read thinkgear ASCII file
%
% Usage: 
% >> data = neurosky_read_thinkgear( filename );
% >> data = neurosky_read_thinkgear( filename, eventfilename );
%
% Input:
%   filename      - filename in the thinkgear format
%   eventfilename - optional event file name containing event latencies
%                   and labels
%
% Output:
%   data  - structure containing the following fields
%           data.srate      = sampling rate
%           data.valuesint  = data values as 8-bit integer
%           data.valuesvolt = data values in volt. Note that
%                             data.valuevolt = data.valuesint/1023*6-3
%           data.timevals   = time values. This is not reliable as this
%                             correspond at the time at which the data
%                             arrived on the port (USB or bluetooth).
%           data.quality    = quality value (1 every second)
%           data.battery    = battery level (1 every second)
%           data.e_sense    = e_sense values (2 values, 1 every second)
%                             this is an array of size 2 x n_seconds
%           data.power      = power values with one value per frequency
%                             band (6 values, 1 every second)
%                             this is an array of size 6 x n_seconds
%           data.databounds = data index boundary at which the 1-second
%                             segments are extracted
%           data.baddataflag = same length as above indexing the bad
%                              portions of data. Contains 0 for good
%                              data segment and 1 for bad ones. To obtain 
%                              this measure detect the presence 0s in the 
%                              attention measure.
%           data.event.type = data event type (only present when the 
%                             eventfilename is present).
%           data.event.latency = data event latency in data point 

% Copyright (c) 2008-2009 NeuroSky, Inc. All Rights Reserved

function [out] = neurosky_read_thinkgear( filename, eventfilename );

    disp('Reading data...');
    
    % load event file first (because much smaller)
    % so if cannot be found, the user does not have to wait
    % -----------------------------------------------------
    if nargin > 1
        events = loadtxt( eventfilename, 'verbose', 'off', 'convert', 'off');
    end;
    % now load the data file
    %array = loadtxt( filename, 'verbose', 'off', 'convert', 'off');
    array = loadtxt( filename, 'verbose', 'off', 'convert', 'off');
    if 0
        f = fopen( filename, 'r');
        if f == -1, error('File not found'); end;
        C = textscan(f, '%s %s %s %s %s %s %s %s %s %s %s');
        fclose(f);
        array = {};
        for i=1:length(C)
            array(1:length(C{i}),i) = C{i};
        end;
    end;
    
    disp('Decoding data...');
    
    allcodes = array(:,2)';
    try codes    = unique(allcodes);
    catch, error('File format problem: are you sure you used the right file format (there are 2 of them)?');
    end;
    if allcodes{1}(1) == '['
        allcodes = cellfun(@(x) x(2:end-1), allcodes, 'UniformOutput',false);
    end;
    
    % extract data
    % ------------
    indvals   = strmatch('80', allcodes, 'exact');
    if ~isempty(indvals)
        if isempty(array{indvals(1),6})
             indcorrect = 1;
        else indcorrect = 0;
        end;
        out.valuesvolt = cellfun(@str2num, array(indvals,6-indcorrect));
        out.valuesint  = 256*cellfun(@hex2dec, array(indvals,4-indcorrect)) + cellfun(@hex2dec, array(indvals,5-indcorrect));
    else
        indvals   = strmatch('06', allcodes, 'exact');        
        if isempty(array{indvals(1),6})
             indcorrect = -1;
        else indcorrect = 0;
        end;
        out.valuesvolt = cellfun(@str2num, array(indvals,5-indcorrect));
        out.valuesint  = cellfun(@hex2dec, array(indvals,4-indcorrect));
    end;
    out.srate    = 128;

    % extract time
    % ------------
    times         = array(indvals,1);
    out.timevals  = cellfun(@reformat_time, times);
    %figure; plot(timesval(400:end));
    %tmprate = 1/mean(timesval(401:end)-timesval(400:end-1)); % 128 Hz
    
    % extract events
    % --------------
    if nargin > 1
        latencies = cellfun(@reformat_time, events(:,1));
        for i = 1:size(events,1)
            [tmp index] = min( abs(latencies(i)-out.timevals) );
            out.event(i).latency = index;
            out.event(i).type    = events{i,2};
        end;
    end;

    % extract e_sense levels
    % ----------------------
    inds   = strmatch('05', allcodes, 'exact'); % meditation
    inds2  = strmatch('04', allcodes, 'exact'); % meditation
    out.e_sense = [];
    if length(inds) ~= length(inds2), disp('Warning: Corrupted e-sense data, ignoring it'); 
    else
        if ~isempty(inds)
            out.e_sense  = zeros(1,length(inds));
            for i = 1:length(inds)
                tmpind = find(inds(i)+1 == indvals); % next index should be data
                                                     % associate this point to it
                if isempty(tmpind), tmpind = find(inds(i)+2 == indvals); end;
                if isempty(tmpind), tmpind = find(inds(i)+3 == indvals); end;
                if isempty(tmpind), tmpind = find(inds(i)+4 == indvals); end;
                if isempty(tmpind), error('Cannot find data index next to attention/meditation measure'); end;
                measure_ind(i)     = tmpind;
                out.e_sense(1,i)   = hex2dec(array{inds2(i), 4-indcorrect}); % inds2 is attention 
                out.e_sense(2,i)   = hex2dec(array{inds(i) , 4-indcorrect}); % current index is meditation 
            end;
        end;
    end;
    
    % extract battery level
    % ---------------------
    inds   = strmatch('01', allcodes, 'exact'); % baterry
    if ~isempty(inds)
        out.battery  = zeros(1,length(inds));
        for i = 1:length(inds)
            out.battery(i)     = hex2dec(array{inds(i), 4-indcorrect}); % index 3 back is quality 
        end;
    else
        out.battery = [];
    end;
    
    % extract quality level
    % ---------------------
    inds   = strmatch('02', allcodes, 'exact'); % baterry
    if ~isempty(inds)
        out.quality  = zeros(1,length(inds));
        for i = 1:length(inds)
            out.quality(i)     = hex2dec(array{inds(i), 4-indcorrect}); % index 3 back is quality 
        end;
    else
        out.quality = [];
    end;
    
    % extract power
    % -------------
    inds   = strmatch('81', allcodes, 'exact'); % power
    if isempty(inds)
        inds   = strmatch('83', allcodes, 'exact'); % power
    end;
    if ~isempty(inds)
        measures_ind    = zeros(size(inds));
        lenpower        = hex2dec(array(inds(1),3));
        out.power       = zeros(size(array,2)-lenpower-3,length(inds));
        try,
            for i = 1:length(inds)
                out.power(:,i)  = cellfun(@str2num,array(inds(i), lenpower+4-indcorrect:end)); % current index contains power
            end;
        catch,
            out.power = [];
            disp('Warning: encountered problem reading power');
        end;
    else
        out.power = [];
    end;
    
    % find data boundaries to extract 
    % 1-s epoch (128 sample values)
    % ------------------------------
    inds           = strmatch('07', allcodes, 'exact');
    if isempty(inds)
        inds       = strmatch('00', allcodes, 'exact');
    end;
    out.databounds = zeros(size(inds));
    for i = 1:length(inds)
        tmpind = find(inds(i)+1 == indvals);
        if isempty(tmpind), error('Cannot find data index for discontinuity'); end;
        out.databounds(i) = tmpind;
    end;
    
    % find bad portions of data
    % -------------------------
    out.baddataflag = out.e_sense(1,:) == 0;
    zeroflag = 0;
    indtmp = find(out.e_sense(1,4:end) == 40)+3;
    for i = 1:length(indtmp)
        if out.e_sense(1,indtmp(i)-1) == 0
            out.baddataflag(1,indtmp(i)-1) = 0;
            out.baddataflag(1,indtmp(i)-2) = 0;
            out.baddataflag(1,indtmp(i)-3) = 0;
        end;    
    end;
    
    % remove data boundaries not associated with any measure
    % ------------------------------------------------------
    nrm = 0;
    if length(out.databounds) >= 2
        if measure_ind(1) < out.databounds(2), nrm = 1; end;
        if measure_ind(1) < out.databounds(1), nrm = 2; end;
        if nrm
            disp('Warning first output values (e_sense, power etc...)');
            disp('are not associated with a data segment, removing them');
            disp(' ');
            if ~isempty(out.battery) out.battery(1:nrm)     = []; end;
            if ~isempty(out.quality) out.quality(1:nrm)     = []; end;
            if ~isempty(out.e_sense) out.e_sense(:,1:nrm)   = []; end;
            if ~isempty(out.power)   out.power(:,1:nrm)     = []; end;
            out.baddataflag(1:nrm) = [];
            measure_ind(1:nrm) = [];
        end;
    end;
    
    % detect inconsistencies
    % ----------------------
    maxlen = min(length(measure_ind), length(out.databounds));
    ind = find( measure_ind(1:maxlen)'-out.databounds(1:maxlen) < 180 );
    if ~isempty(ind)
        fprintf('An inconsistency has been detected at segment %d\n', ind(1));
    end;
    
    inds = find(out.databounds+128 > measure_ind(end));
    if ~isempty(inds)
        fprintf('Warning: %d data segment at the end of the file\n', length(inds));
        disp('do(es) not have any measure associated to them, removing them(it)');
        disp(' ');
        out.databounds(inds) = [];
    end;
    if length(out.databounds) >= 2 & length(out.databounds) ~= size(out.e_sense, 2)
        error('Warning: could not match every measure with a data boundary');
    end;
    
function val = reformat_time(str)

ind = find(str == '.');
if isempty(ind)
     val = str2num(str(1:end-1));
else
    if str(end) == ':'
        val = str2num(str(ind-5:end-1));    
    else
        val = str2num(str(ind-5:end));    
    end;
end;