% FILESTREAM file streaming object for streamlog NeuroSky files
%
%    FILESTREAM('key1', val1, 'key2', val2)
%    'key' can be any of the following
%
%    'file' - [string] automatically open the file provided as input.
%    'refreshrate' - [integer] refreshrate of the object in second. At each
%                 refreshrate period, in continuous mode, a number of 
%                 bytes (defined in 'bytespersecond' is read).
%    'fig' - current figure handle. This is used to create menus for
%                 opening data files.
%    'firmware' - string describing the firmware ('1.5', '1.7', '1.79' and
%                 'FPGA - board' or 'default')
%    'popup' - ['on'|'off'] 'on' pops up a file opening window. Default
%                 is 'off'.
%    'filepos' - [integer or string] starting file position in number of 
%                 bytes. Default is 'continuous' meaning that the data is
%                 delivered continuously assuming a number of bytes per
%                 seconds as defined below.
%    'bytespersecond' - [integer] number of bytes per second for continuous
%                 streaming. Default is 1024 (8 bytes per packet at 128 Hz)
% 
%    The function call returns an instance to the object.

% Copyright (c) 2008-2009 NeuroSky, Inc. All Rights Reserved

function this = filestream(varargin);

    if ~isempty(varargin) && isstruct(varargin{1})
        opt = varargin{1};
    else
        opt = struct(varargin{:});
    end;
        
    if ~isfield(opt, 'fig'),                opt.fig = []; end;
    if ~isfield(opt, 'file'),               opt.file = []; end;
    if ~isfield(opt, 'popup'),              opt.popup = 'off'; end;
    if ~isfield(opt, 'instancename'),       opt.instancename = ''; end;
    if ~isfield(opt, 'filepos'),            opt.filepos = 'continuous'; end; % position in seconds
    if ~isfield(opt, 'bytespersecond'),     opt.bytespersecond = 1024; end; % position in seconds
    if ~isfield(opt, 'refreshrate'),        opt.refreshrate = 0; end;
    if ~isfield(opt, 'streambuffervar'),    opt.streambuffervar = 'streambuffer'; end;
    if ~isfield(opt, 'matlabfilefield'),    opt.matlabfilefield = 'dataall'; end;
    
    this.instancename       = opt.instancename;
    this.filepos            = opt.filepos;
    this.refreshrate        = opt.refreshrate;
    this.bytespersecond     = opt.bytespersecond;
    this.streambuffervar    = opt.streambuffervar;
    this.matlabfilefield    = opt.matlabfilefield;
    this.filebytesize       = 1000000;
    %this.params             = [];
    headsetinfoobj          = headsetinfo(opt);

    tic;
    this.fileid_replay    = [];    
    this.tmptime          = toc;
    
    % create menus
    % ------------
    if ~isempty(opt.fig)
        m_f = findobj('parent', opt.fig, 'label', 'File');
        if ~isempty(m_f)
            cb_replay       = [ this.instancename ' = replay(' this.instancename ', []);' ];
            uimenu(m_f, 'Label', 'Replay data file', 'callback', cb_replay);
        end;
        drawnow;
    end;
    
    % create class
    this = class(this, 'filestream', headsetinfoobj);
    
    % load file
    if ~isempty(opt.file) | strcmpi(opt.popup, 'on')
        this = replay(this, opt.file);
        
        % position in file (some bytes may be lost to achieve precise positioning)
        if ~strcmpi(opt.filepos, 'continuous')
            if isinf(opt.filepos)
                opt.filepos = 22118400; % 24 hours at 256 Hz
            end;
            this.filepos = round(opt.filepos*this.bytespersecond);
            this = filestream_process(this);
        end;
    end;
    
    