% HEADSETINFO get headset information.
%
%    HEADSETINFO('key1', val1, 'key2', val2)
%    'key' can be any of the following
%
%    'popup' - ['on'|'off'] 'on' pops up a serial port opening window. 
%                 Default is 'on'.
%    'port' - [ 'COMX' ] if not empty, automatically connects to port COMX,
%                 X defining the port. 'auto' will detect automatically 
%                 which port is connected. Default is empty.
%    'refreshrate' - [integer] refreshrate of the object in second. At each
%                 refreshrate period, the port buffer is flushed.
%    'headset' - string describing the headset in mindsetinfo folder
%    'headsetoption' - integer indicating the headset option
%    'inputbuffersize' - [integer] maximum number of bytes in buffer.
%                 Default is 2048.
%    'connectopenspeed' - [integer] speed in baud to open port. Default is
%                 9600.
%    'connectstreamspeed' - [integer] speed in baud for data streaming.
%                 Default is 57200.
%    'commands' - [cell] commands to send to headset after opening the port
%                 but before changing to speed to stream data. Default is 
%                 {}. For the 1.79 firmware, the default is { 18 15 }. Note
%                 defining a firmware other than default, automatically set 
%                 this parameter.

% Copyright (c) 2008-2010 NeuroSky, Inc. All Rights Reserved

function this = headsetinfo(varargin);

    if ~isempty(varargin) && isstruct(varargin{1})
        opt = varargin{1};
    else
        opt = struct(varargin{:});
    end;
        
    if isfield(opt, 'firmware'),           opt.headset       = opt.firmware; end;
    if ~isfield(opt, 'headset'),           opt.headset       = 'mindset'; end; % use custom to enter parameter above
    if ~isfield(opt, 'headsetoption'),     opt.headsetoption = 1; end; % use custom to enter parameter above
    this.headset           = opt.headset; %1.5;
    this.headsetoption     = opt.headsetoption; %1.5;
    if isnumeric(this.headset), this.headset = 'mindbuilder'; end;
    
    % load all the configuration text files
    % -------------------------------------
    this.allformats         = [];
    if isdeployed
        str = ctfroot;
        inds = find(str == filesep);
        folder = str(1:inds(end)-1);
    else
        folder = which('selectinfo');
        folder = fileparts(folder);
    end;
    allfiles = dir(fullfile(folder, [ 'mindsetinfo' filesep '*.txt' ]));
    for ind = 1:length(allfiles)
        structure = loadinfo(fullfile(folder, 'mindsetinfo', allfiles(ind).name));
        fields = fieldnames(structure);
        for f = 1:length(fields)
            tmpval = getfield(structure, fields{f});
            this.allformats = setfield(this.allformats, { ind }, fields{f}, tmpval);
        end;
    end;
    
    % no Graphical interface, check format name
    % -----------------------------------------
    this.params = selectinfo(this.allformats, this.headset, this.headsetoption);
    
    if ~isfield(opt, 'opening_speed'),       opt.opening_speed      = this.params.opening_speed; end;
    if ~isfield(opt, 'streaming_speed'),     opt.streaming_speed    = this.params.streaming_speed; end;
    if ~isfield(opt, 'connect_command_1'),   opt.connect_command_1  = this.params.connect_command_1; end;
    if ~isfield(opt, 'connect_command_2'),   opt.connect_command_2  = this.params.connect_command_2; end;
    if ~isfield(opt, 'gain'),                opt.gain               = this.params.gain; end;
    if ~isfield(opt, 'lower_bit_level'),     opt.lower_bit_level    = this.params.lower_bit_level; end;
    if ~isfield(opt, 'upper_bit_level'),     opt.upper_bit_level    = this.params.upper_bit_level; end;
    if ~isfield(opt, 'sampling_rate'),       opt.sampling_rate      = this.params.sampling_rate; end;
    if ~isfield(opt, 'volt_reference'),      opt.volt_reference     = this.params.volt_reference; end;
    
    if ~isfield(opt, 'connectinterface'),   opt.connectinterface   = 'complex'; end;
    if ~isfield(opt, 'port'),               opt.port               = 'gui'; end;
    if ~isfield(opt, 'inputbuffersize'),    opt.inputbuffersize    = 2048; end;
    if ~isfield(opt, 'popup'),              opt.popup              = 'on'; end;
    if ~isfield(opt, 'instancename'),       opt.instancename       = ''; end;
    
    this.params.opening_speed       = opt.opening_speed;
    this.params.streaming_speed     = opt.streaming_speed;
    this.params.connect_command_1   = opt.connect_command_1;
    this.params.connect_command_2   = opt.connect_command_2;
    this.params.gain                = opt.gain;
    this.params.lower_bit_level     = opt.lower_bit_level;
    this.params.upper_bit_level     = opt.upper_bit_level;
    this.params.sampling_rate       = opt.sampling_rate;
    this.params.volt_reference      = opt.volt_reference;
    
    this.connectinterface   = opt.connectinterface;
    this.instancename       = opt.instancename;
    this.port               = opt.port;
    
    this = class(this, 'headsetinfo');
    